@@ -71,6 +71,7 @@
 
 #include "gstgnomevfssrc.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -94,13 +95,6 @@
 GST_DEBUG_CATEGORY_STATIC (gnomevfssrc_debug);
 #define GST_CAT_DEFAULT gnomevfssrc_debug
 
-static const GstElementDetails gst_gnome_vfs_src_details =
-GST_ELEMENT_DETAILS ("GnomeVFS Source",
-    "Source/File",
-    "Read from any GnomeVFS-supported file",
-    "Bastien Nocera <hadess@hadess.net>\n"
-    "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
-
 static GStaticMutex count_lock = G_STATIC_MUTEX_INIT;
 static gint ref_count = 0;
 static gboolean vfs_owner = FALSE;
@@ -184,7 +178,11 @@ gst_gnome_vfs_src_base_init (gpointer g_class)
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&srctemplate));
-  gst_element_class_set_details (element_class, &gst_gnome_vfs_src_details);
+  gst_element_class_set_details_simple (element_class,
+      "GnomeVFS Source", "Source/File",
+      "Read from any GnomeVFS-supported file",
+      "Bastien Nocera <hadess@hadess.net>, "
+      "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
 
   GST_DEBUG_CATEGORY_INIT (gnomevfssrc_debug, "gnomevfssrc", 0,
       "Gnome-VFS Source");
@@ -608,8 +606,8 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
 
   src = GST_GNOME_VFS_SRC (basesrc);
 
-  GST_DEBUG ("now at %llu, reading from %lld, size %u", src->curoffset, offset,
-      size);
+  GST_DEBUG ("now at %" G_GINT64_FORMAT ", reading from %" G_GUINT64_FORMAT
+      ", size %u", src->curoffset, offset, size);
 
   /* seek if required */
   if (G_UNLIKELY (src->curoffset != offset)) {
@@ -626,7 +624,7 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
   }
 
   buf = gst_buffer_try_new_and_alloc (size);
-  if (G_UNLIKELY (buf == NULL && size == 0)) {
+  if (G_UNLIKELY (buf == NULL)) {
     GST_ERROR_OBJECT (src, "Failed to allocate %u bytes", size);
     return GST_FLOW_ERROR;
   }
