@@ -38,36 +38,47 @@
 
 #include "gstffmpegcolorspace.h"
 #include "gstffmpegcodecmap.h"
+#include <gst/video/video.h>
 
 GST_DEBUG_CATEGORY (ffmpegcolorspace_debug);
 #define GST_CAT_DEFAULT ffmpegcolorspace_debug
 GST_DEBUG_CATEGORY (ffmpegcolorspace_performance);
 
-/* elementfactory information */
-static const GstElementDetails ffmpegcsp_details =
-GST_ELEMENT_DETAILS ("FFMPEG Colorspace converter",
-    "Filter/Converter/Video",
-    "Converts video from one colorspace to another",
-    "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
+#define FFMPEGCSP_VIDEO_CAPS						\
+  "video/x-raw-yuv, width = "GST_VIDEO_SIZE_RANGE" , "			\
+  "height="GST_VIDEO_SIZE_RANGE",framerate="GST_VIDEO_FPS_RANGE","	\
+  "format= (fourcc) { I420 , NV12 , NV21 , YV12 , YUY2 , Y42B , Y444 , YUV9 , YVU9 , Y41B , Y800 , Y8 , GREY , Y16 , UYVY , YVYU , IYU1 , v308 , AYUV } ;" \
+  GST_VIDEO_CAPS_RGB";"							\
+  GST_VIDEO_CAPS_BGR";"							\
+  GST_VIDEO_CAPS_RGBx";"						\
+  GST_VIDEO_CAPS_xRGB";"						\
+  GST_VIDEO_CAPS_BGRx";"						\
+  GST_VIDEO_CAPS_xBGR";"						\
+  GST_VIDEO_CAPS_RGBA";"						\
+  GST_VIDEO_CAPS_ARGB";"						\
+  GST_VIDEO_CAPS_BGRA";"						\
+  GST_VIDEO_CAPS_ABGR";"						\
+  GST_VIDEO_CAPS_RGB_16";"						\
+  GST_VIDEO_CAPS_RGB_15";"						\
+  GST_VIDEO_CAPS_GRAY8";"						\
+  GST_VIDEO_CAPS_GRAY16("BIG_ENDIAN")";"				\
+  GST_VIDEO_CAPS_GRAY16("LITTLE_ENDIAN")";"
 
+static GstStaticPadTemplate gst_ffmpegcsp_src_template =
+GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (FFMPEGCSP_VIDEO_CAPS)
+    );
 
-/* Stereo signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
+static GstStaticPadTemplate gst_ffmpegcsp_sink_template =
+GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (FFMPEGCSP_VIDEO_CAPS)
+    );
 
-enum
-{
-  ARG_0,
-};
-
-static GType gst_ffmpegcsp_get_type (void);
-
-static void gst_ffmpegcsp_base_init (GstFFMpegCspClass * klass);
-static void gst_ffmpegcsp_class_init (GstFFMpegCspClass * klass);
-static void gst_ffmpegcsp_init (GstFFMpegCsp * space);
+GType gst_ffmpegcsp_get_type (void);
 
 static gboolean gst_ffmpegcsp_set_caps (GstBaseTransform * btrans,
     GstCaps * incaps, GstCaps * outcaps);
@@ -75,60 +86,35 @@ static gboolean gst_ffmpegcsp_get_unit_size (GstBaseTransform * btrans,
     GstCaps * caps, guint * size);
 static GstFlowReturn gst_ffmpegcsp_transform (GstBaseTransform * btrans,
     GstBuffer * inbuf, GstBuffer * outbuf);
-#if 0
-static GstFlowReturn gst_ffmpegcsp_transform_ip (GstBaseTransform * btrans,
-    GstBuffer * inbuf);
-#endif
 
-static GstPadTemplate *sinktempl, *srctempl;
-static GstElementClass *parent_class = NULL;
-
-/*static guint gst_ffmpegcsp_signals[LAST_SIGNAL] = { 0 }; */
+static GQuark _QRAWRGB;         /* "video/x-raw-rgb" */
+static GQuark _QRAWYUV;         /* "video/x-raw-yuv" */
+static GQuark _QALPHAMASK;      /* "alpha_mask" */
 
 /* copies the given caps */
 static GstCaps *
 gst_ffmpegcsp_caps_remove_format_info (GstCaps * caps)
 {
-  int i;
-  GstStructure *structure;
-  GstCaps *rgbcaps;
-  GstCaps *graycaps;
+  GstStructure *yuvst, *rgbst, *grayst;
 
+  /* We know there's only one structure since we're given simple caps */
   caps = gst_caps_copy (caps);
 
-  for (i = 0; i < gst_caps_get_size (caps); i++) {
-    structure = gst_caps_get_structure (caps, i);
+  yuvst = gst_caps_get_structure (caps, 0);
 
-    gst_structure_set_name (structure, "video/x-raw-yuv");
-    gst_structure_remove_field (structure, "format");
-    gst_structure_remove_field (structure, "endianness");
-    gst_structure_remove_field (structure, "depth");
-    gst_structure_remove_field (structure, "bpp");
-    gst_structure_remove_field (structure, "red_mask");
-    gst_structure_remove_field (structure, "green_mask");
-    gst_structure_remove_field (structure, "blue_mask");
-    gst_structure_remove_field (structure, "alpha_mask");
-    gst_structure_remove_field (structure, "palette_data");
-  }
+  gst_structure_set_name (yuvst, "video/x-raw-yuv");
+  gst_structure_remove_fields (yuvst, "format", "endianness", "depth",
+      "bpp", "red_mask", "green_mask", "blue_mask", "alpha_mask",
+      "palette_data", NULL);
 
-  gst_caps_do_simplify (caps);
-  rgbcaps = gst_caps_copy (caps);
+  rgbst = gst_structure_copy (yuvst);
+  gst_structure_set_name (rgbst, "video/x-raw-rgb");
 
-  for (i = 0; i < gst_caps_get_size (rgbcaps); i++) {
-    structure = gst_caps_get_structure (rgbcaps, i);
+  grayst = gst_structure_copy (rgbst);
+  gst_structure_set_name (grayst, "video/x-raw-gray");
 
-    gst_structure_set_name (structure, "video/x-raw-rgb");
-  }
-  graycaps = gst_caps_copy (caps);
-
-  for (i = 0; i < gst_caps_get_size (graycaps); i++) {
-    structure = gst_caps_get_structure (graycaps, i);
-
-    gst_structure_set_name (structure, "video/x-raw-gray");
-  }
-
-  gst_caps_append (caps, graycaps);
-  gst_caps_append (caps, rgbcaps);
+  gst_caps_append_structure (caps, rgbst);
+  gst_caps_append_structure (caps, grayst);
 
   return caps;
 }
@@ -137,13 +123,13 @@ gst_ffmpegcsp_caps_remove_format_info (GstCaps * caps)
 static gboolean
 gst_ffmpegcsp_structure_is_alpha (GstStructure * s)
 {
-  const gchar *name;
+  GQuark name;
 
-  name = gst_structure_get_name (s);
+  name = gst_structure_get_name_id (s);
 
-  if (g_str_equal (name, "video/x-raw-rgb")) {
-    return gst_structure_has_field (s, "alpha_mask");
-  } else if (g_str_equal (name, "video/x-raw-yuv")) {
+  if (name == _QRAWRGB) {
+    return gst_structure_id_has_field (s, _QALPHAMASK);
+  } else if (name == _QRAWYUV) {
     guint32 fourcc;
 
     if (!gst_structure_get_fourcc (s, "format", &fourcc))
@@ -165,38 +151,28 @@ gst_ffmpegcsp_transform_caps (GstBaseTransform * btrans,
   GstCaps *template;
   GstCaps *tmp, *tmp2;
   GstCaps *result;
-  guint i, n;
-  gboolean is_alpha;
   GstStructure *s;
   GstCaps *alpha, *non_alpha;
 
-  s = gst_caps_get_structure (caps, 0);
-  is_alpha = gst_ffmpegcsp_structure_is_alpha (s);
-
-  template = gst_ffmpegcsp_codectype_to_caps (CODEC_TYPE_VIDEO, NULL);
-  result = gst_caps_intersect (caps, template);
+  template = gst_static_pad_template_get_caps (&gst_ffmpegcsp_src_template);
+  result = gst_caps_copy (caps);
 
   /* Get all possible caps that we can transform to */
   tmp = gst_ffmpegcsp_caps_remove_format_info (caps);
   tmp2 = gst_caps_intersect (tmp, template);
-  gst_caps_unref (template);
   gst_caps_unref (tmp);
   tmp = tmp2;
 
   /* Now move alpha formats to the beginning if caps is an alpha format
    * or at the end if caps is no alpha format */
-  n = gst_caps_get_size (tmp);
-
   alpha = gst_caps_new_empty ();
   non_alpha = gst_caps_new_empty ();
 
-  for (i = 0; i < n; i++) {
-    s = gst_caps_get_structure (tmp, i);
-
+  while ((s = gst_caps_steal_structure (tmp, 0))) {
     if (gst_ffmpegcsp_structure_is_alpha (s))
-      gst_caps_append_structure (alpha, gst_structure_copy (s));
+      gst_caps_append_structure (alpha, s);
     else
-      gst_caps_append_structure (non_alpha, gst_structure_copy (s));
+      gst_caps_append_structure (non_alpha, s);
   }
 
   s = gst_caps_get_structure (caps, 0);
@@ -352,39 +328,27 @@ invalid_out_caps:
   }
 }
 
-static GType
-gst_ffmpegcsp_get_type (void)
-{
-  static GType ffmpegcsp_type = 0;
-
-  if (!ffmpegcsp_type) {
-    static const GTypeInfo ffmpegcsp_info = {
-      sizeof (GstFFMpegCspClass),
-      (GBaseInitFunc) gst_ffmpegcsp_base_init,
-      NULL,
-      (GClassInitFunc) gst_ffmpegcsp_class_init,
-      NULL,
-      NULL,
-      sizeof (GstFFMpegCsp),
-      0,
-      (GInstanceInitFunc) gst_ffmpegcsp_init,
-    };
-
-    ffmpegcsp_type = g_type_register_static (GST_TYPE_BASE_TRANSFORM,
-        "GstFFMpegCsp", &ffmpegcsp_info, 0);
-  }
-
-  return ffmpegcsp_type;
-}
+GST_BOILERPLATE (GstFFMpegCsp, gst_ffmpegcsp, GstVideoFilter,
+    GST_TYPE_VIDEO_FILTER);
 
 static void
-gst_ffmpegcsp_base_init (GstFFMpegCspClass * klass)
+gst_ffmpegcsp_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class, srctempl);
-  gst_element_class_add_pad_template (element_class, sinktempl);
-  gst_element_class_set_details (element_class, &ffmpegcsp_details);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_ffmpegcsp_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_ffmpegcsp_sink_template));
+
+  gst_element_class_set_details_simple (element_class,
+      "FFMPEG Colorspace converter", "Filter/Converter/Video",
+      "Converts video from one colorspace to another",
+      "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
+
+  _QRAWRGB = g_quark_from_string ("video/x-raw-rgb");
+  _QRAWYUV = g_quark_from_string ("video/x-raw-yuv");
+  _QALPHAMASK = g_quark_from_string ("alpha_mask");
 }
 
 static void
@@ -401,13 +365,9 @@ gst_ffmpegcsp_finalize (GObject * obj)
 static void
 gst_ffmpegcsp_class_init (GstFFMpegCspClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstBaseTransformClass *gstbasetransform_class;
-
-  gobject_class = (GObjectClass *) klass;
-  gstbasetransform_class = (GstBaseTransformClass *) klass;
-
-  parent_class = g_type_class_peek_parent (klass);
+  GObjectClass *gobject_class = (GObjectClass *) klass;
+  GstBaseTransformClass *gstbasetransform_class =
+      (GstBaseTransformClass *) klass;
 
   gobject_class->finalize = gst_ffmpegcsp_finalize;
 
@@ -418,18 +378,13 @@ gst_ffmpegcsp_class_init (GstFFMpegCspClass * klass)
       GST_DEBUG_FUNCPTR (gst_ffmpegcsp_get_unit_size);
   gstbasetransform_class->transform =
       GST_DEBUG_FUNCPTR (gst_ffmpegcsp_transform);
-#if 0
-  gstbasetransform_class->transform_ip =
-      GST_DEBUG_FUNCPTR (gst_ffmpegcsp_transform_ip);
-#endif
 
   gstbasetransform_class->passthrough_on_same_caps = TRUE;
 }
 
 static void
-gst_ffmpegcsp_init (GstFFMpegCsp * space)
+gst_ffmpegcsp_init (GstFFMpegCsp * space, GstFFMpegCspClass * klass)
 {
-  gst_base_transform_set_qos_enabled (GST_BASE_TRANSFORM (space), TRUE);
   space->from_pixfmt = space->to_pixfmt = PIX_FMT_NB;
   space->palette = NULL;
 }
@@ -482,16 +437,6 @@ beach:
   return ret;
 }
 
-#if 0
-/* FIXME: Could use transform_ip to implement endianness swap type operations */
-static GstFlowReturn
-gst_ffmpegcsp_transform_ip (GstBaseTransform * btrans, GstBuffer * inbuf)
-{
-  /* do nothing */
-  return GST_FLOW_OK;
-}
-#endif
-
 static GstFlowReturn
 gst_ffmpegcsp_transform (GstBaseTransform * btrans, GstBuffer * inbuf,
     GstBuffer * outbuf)
@@ -547,25 +492,21 @@ not_supported:
   }
 }
 
-gboolean
-gst_ffmpegcolorspace_register (GstPlugin * plugin)
+static gboolean
+plugin_init (GstPlugin * plugin)
 {
-  GstCaps *caps;
-
   GST_DEBUG_CATEGORY_INIT (ffmpegcolorspace_debug, "ffmpegcolorspace", 0,
       "FFMPEG-based colorspace converter");
   GST_DEBUG_CATEGORY_GET (ffmpegcolorspace_performance, "GST_PERFORMANCE");
 
-  /* template caps */
-  caps = gst_ffmpegcsp_codectype_to_caps (CODEC_TYPE_VIDEO, NULL);
-
-  /* build templates */
-  srctempl = gst_pad_template_new ("src",
-      GST_PAD_SRC, GST_PAD_ALWAYS, gst_caps_copy (caps));
-
-  /* the sink template will do palette handling as well... */
-  sinktempl = gst_pad_template_new ("sink", GST_PAD_SINK, GST_PAD_ALWAYS, caps);
+  avcodec_init ();
 
   return gst_element_register (plugin, "ffmpegcolorspace",
       GST_RANK_NONE, GST_TYPE_FFMPEGCSP);
 }
+
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "ffmpegcolorspace",
+    "colorspace conversion copied from FFMpeg " FFMPEG_VERSION,
+    plugin_init, VERSION, "LGPL", "FFMpeg", "http://ffmpeg.sourceforge.net/")
